
(* This calculates the numerator of the series \sum n^j q^j. *)

Euler[j_] := Euler[j] = If[j == 0, 1, Expand[q((1 - q)D[Euler[j - 1], q] + j Euler[j - 1])]]

(* Here if y={y[[1]],...,y[[d]]} (a la Mathematica) then it yields the polynomial \product (y[i]x + 1)
which is the Hilbert polynomial of the image of \pi_\lambda if the y[[i]] are
the c_\lambda(\beta_i) and the \beta_i is an enumeration of the roots for 
which <\lambda,\alpha> > 0. *)

pol[y_] := Product[x*y[[i]]+1,{i,1,Length[y]}]

(*This calculates the numerator of the Hilbert series of the image of \pi_\lambda of G/P when the y[[i]] are
the c_\lambda(\beta_i) and the \beta_i is an enumeration of the roots for which <\lambda,\alpha> > 0.*)

NumSer[y_] := Module[{g=0,u,d=Length[y]},u=pol[y]; 
For[i=0,i<=d,i++,g=Expand[g+Coefficient[u,x,i] Euler[i](1-q)^(d-i)]];
g]

(* This inputs the code to calculate the root systems used in the next functions. Make sure that this file
is in the same directory as hilbcalc.m *)
<<roots.m
(*This calculates the vector of c_\lambda(\beta_i) as in poly above *)

Mkx[Type_,rank_,a_] :=	Module[{L,i,r,x,y,z={}},
			If[Length[a]!=rank,Print["The weight must have",rank," entries."];Return[]];
			If[Type == "A",r = Table[1,{rank}];
				L=ARoots[rank];
				For[i=1,i<=Length[L],i++,x=a.L[[i]];
				If[x != 0,z=Append[z, x/(r.L[[i]])]]];
				Return[z]];
			If[Type == "B",r = Table[1,{rank}];
			    r[[rank]] = 1/2;
			    y=a; y[[rank]]=a[[rank]]/2;
				L=BRoots[rank];
				For[i=1,i<=Length[L],i++,x=y.L[[i]];
				If[x != 0,z=Append[z, x/(r.L[[i]])]]];
				Return[z]];
			If[Type == "C",r = Table[1,{rank}];
			    r[[rank]] = 2;
			    y=a; y[[rank]]=2 a[[rank]];
				L=CRoots[rank];
				For[i=1,i<=Length[L],i++,x=y.L[[i]];
				If[x != 0,z=Append[z, x/(r.L[[i]])]]];
				Return[z]];
			If[Type == "D",r = Table[1,{rank}];
				L=DRoots[rank];
				For[i=1,i<=Length[L],i++,x=a.L[[i]];
				If[x != 0,z=Append[z, x/(r.L[[i]])]]];
				Return[z]];
			If[Type == "E",If[(rank < 6)||(rank > 8),Print["Only E6,E7,E8 are allowed"];Return[]];
				r = Table[1,{rank}];
				L=ERoots[rank];
				For[i=1,i<=Length[L],i++,x=a.L[[i]];
				If[x != 0,z=Append[z, x/(r.L[[i]])]]];
				Return[z]];
			If[Type == "F",If[rank != 4,Print["Only F_4 exists"];Return[]];
			    L=F4Roots;r = {1,1,1/2,1/2};
			    y={a[[1]],a[[2]],a[[3]]/2,a[[4]]/2};
				For[i=1,i<=Length[L],i++,x=y.L[[i]];
				If[x != 0,z=Append[z,x/(r.L[[i]])]]];
				Return[z]];
			If[Type == "G",If[rank !=2, Print["Only G_2 exists"];Return[]];
				L=G2Roots;r={1,3};y={a[[1]],3a[[2]]};
				For[i=1,i<=Length[L],i++,x=y.L[[i]];
				If[x != 0,z=Append[z, x/(r.L[[i]])]]];
				Return[z]]
				]

(* The following use the vector to calculate the degree, Hilbert polynomia and Hilbert series of \pi_a.
Here type is one of "A" through "G" and the highest weight is given in terms of fundamental highest weights. *)				
Deg[Type_,rank_,a_]:= Module[{L,i},
			L=Mkx[Type,rank,a];
			Length[L]! Product[L[[i]],{i,1,Length[L]}]]			
				
HilbPoly[Type_,rank_,a_] := Module[{L,i},L=Mkx[Type,rank,a];
							u=Product[t L[[i]]+1,{i,1,Length[L]}]]

HilbSeries[Type_,rank_,a_] := Module[{L,i,g=0,u},
			L=Mkx[Type,rank,a];d=Length[L];
			u=Product[t L[[i]]+1,{i,1,Length[L]}]; 
			For[i=0,i<=d,i++,g=Expand[g+Coefficient[u,t,i] Euler[i](1-q)^(d-i)]];
			g/(1-q)^(d+1)]

(* The next do the same thing for products here the Type is an arroay {"X","Y",...} with X,Y,... in A-G
and rank is an array of ranks, a is an arroay of highest weights of the correct size.*)			
ProdMkx[Type_,rank_,a_] := Module[{L={},i,j,M},
			If[(Length[Type] != Length[rank]) ||(Length[Type] != Length[a]),
				Print["All of the arrays should heve the same number of enries."];Return[]];
			For[i=1,i<=Length[Type],i++,M=Mkx[Type[[i]],rank[[i]],a[[i]]];
			    For[j=1,j<= Length[M],j++,L=Append[L,M[[j]]]]];
			L]
			
PolyHilbPoly[Type_,rank_,a_] := Module[{L,i},L=ProdMkx[Type,rank,a];
							u=Product[t L[[i]]+1,{i,1,Length[L]}]]

ProdDeg[Type_,rank_,a_]:= Module[{L,i},
			L=ProdMkx[Type,rank,a];
			Length[L]! Product[L[[i]],{i,1,Length[L]}]]
			
ProdHilbSeries[Type_,rank_,a_] := Module[{L,i,g=0,u},
			L=ProdMkx[Type,rank,a];d=Length[L];
			u=Product[t L[[i]]+1,{i,1,Length[L]}]; 
			For[i=0,i<=d,i++,g=Expand[g+Coefficient[u,t,i] Euler[i](1-q)^(d-i)]];
			g/(1-q)^(d+1)]
			  
										